package com.daniminguet.videogameadvisor.service;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.Collection;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.daniminguet.videogameadvisor.dao.VideogameDao;
import com.daniminguet.videogameadvisor.model.Videogame;

@Service
public class VideogameQueryServiceImpl implements VideogameQueryService {
	@Autowired
	private VideogameDao dao;
	
	private Predicate<Videogame> predicate;
	
	@PostConstruct
	public void iniciar() {
		predicate = null;
	}

	@Override
	public Collection<Videogame> ejecutar() {
		//@formatter:off
		return dao.mostrarTodos()
				.stream()
				.filter(predicate)
				.collect(Collectors.toList());
		//@formatter:on
	}

	@Override
	public VideogameQueryService buscarGenero(String... genero) {
		Predicate<Videogame> pBuscarGenero = (vg -> Arrays.stream(genero).anyMatch(vg.getGeneros()::contains));
		predicate = (predicate == null) ? pBuscarGenero : predicate.and(pBuscarGenero);
		return this;
	}

	@Override
	public VideogameQueryService mostrarGeneros(String... generos) {
		Predicate<Videogame> pTodosGeneros = (vg -> Arrays.stream(generos).allMatch(vg.getGeneros()::contains));
		predicate = (predicate == null) ? pTodosGeneros : predicate.and(pTodosGeneros);
		return this;
	}

	@Override
	public VideogameQueryService buscarPorAnyo(String anyo) {
		Predicate<Videogame> pAnyo = (vg -> vg.getFechaSalida().substring(vg.getFechaSalida().length()-4, vg.getFechaSalida().length()-1).equalsIgnoreCase(anyo));
		predicate = (predicate == null) ? pAnyo : predicate.and(pAnyo);
		return this;
	}

	@Override
	public VideogameQueryService buscarEntreAnyos(String desde, String hasta) {
		Predicate<Videogame> pEntreAnyos = (vg -> {
			LocalDate desdeAnyo = LocalDate.of(Integer.parseInt(desde), 1, 1);
			LocalDate hastaAnyo = LocalDate.of(Integer.parseInt(hasta), 1, 3);
			LocalDate videojuego = LocalDate.of(Integer.parseInt(vg.getFechaSalida().substring(vg.getFechaSalida().length()-4, vg.getFechaSalida().length()-1)), 1, 2);
			
			return videojuego.isAfter(desdeAnyo) && videojuego.isBefore(hastaAnyo);
		});
		
		predicate = (predicate == null) ? pEntreAnyos : predicate.and(pEntreAnyos);
		return this;
	}

	@Override
	public VideogameQueryService buscarNombre(String nombre) {
		Predicate<Videogame> pNombre = (vg -> vg.getNombre().toLowerCase().contains(nombre.toLowerCase()));
		predicate = (predicate == null) ? pNombre : predicate.and(pNombre);
		return this;
	}

	@Override
	public VideogameQueryService buscarDesarrollador(String desarrollador) {
		Predicate<Videogame> pDesarrollador = (vg -> vg.getNombre().toLowerCase().contains(desarrollador.toLowerCase()));
		predicate = (predicate == null) ? pDesarrollador : predicate.and(pDesarrollador);
		return this;
	}

	@Override
	public VideogameQueryService buscarPlataforma(String... plataforma) {
		Predicate<Videogame> pBuscarPlataforma = (vg -> Arrays.stream(plataforma).anyMatch(vg.getPlataformas()::contains));
		predicate = (predicate == null) ? pBuscarPlataforma : predicate.and(pBuscarPlataforma);
		return this;
	}

	@Override
	public VideogameQueryService mostrarPlataformas(String... plataformas) {
		Predicate<Videogame> pTodasPlataformas = (vg -> Arrays.stream(plataformas).allMatch(vg.getPlataformas()::contains));
		predicate = (predicate == null) ? pTodasPlataformas : predicate.and(pTodasPlataformas);
		return this;
	}

}
