package com.daniminguet.videogameadvisor.service;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.daniminguet.videogameadvisor.dao.VideogameDao;
import com.daniminguet.videogameadvisor.model.Videogame;

@Service
public class VideogameService {
	@Autowired
	VideogameDao videogameDao;

	@Autowired
	VideogameQueryService queryService;

	public Collection<String> mostrarGeneros() {
		List<String> generos = null;

		// @formatter:off
		generos = videogameDao.mostrarTodos()
				.stream()
				.map(vg -> vg.getGeneros())
				.flatMap(list -> list.stream())
				.distinct()
				.sorted()
				.collect(Collectors.toList());

		// @formatter:on

		return generos;
	}

	public Collection<Videogame> buscarGenero(String... genero) {
		return queryService.buscarGenero(genero).ejecutar();
	}

	public Collection<Videogame> buscarAnyo(String anyo) {
		return queryService.buscarPorAnyo(anyo).ejecutar();
	}

	public Collection<Videogame> buscarEntreAnyos(String desde, String hasta) {
		return queryService.buscarEntreAnyos(desde, hasta).ejecutar();
	}

	public Collection<Videogame> buscarNombre(String nombre) {
		return queryService.buscarNombre(nombre).ejecutar();
	}

	public Collection<Videogame> mostrarTodos() {
		return videogameDao.mostrarTodos();
	}
	
	public Collection<Videogame> buscarDesarrollador(String desarrollador) {
		return queryService.buscarDesarrollador(desarrollador).ejecutar();
	}
	
	public Collection<Videogame> buscarPlataforma(String plataforma) {
		return queryService.buscarPlataforma(plataforma).ejecutar();
	}
	
	public Collection<String> mostrarPlataformas() {
		List<String> plataformas = null;

		// @formatter:off
		plataformas = videogameDao.mostrarTodos()
				.stream()
				.map(vg -> vg.getPlataformas())
				.flatMap(list -> list.stream())
				.distinct()
				.sorted()
				.collect(Collectors.toList());

		// @formatter:on

		return plataformas;
	}
}
